package ai.giskard.web.rest;

import ai.giskard.IntegrationTest;
import ai.giskard.config.ApplicationProperties;
import ai.giskard.domain.Project;
import ai.giskard.repository.ProjectRepository;
import ai.giskard.repository.UserRepository;
import ai.giskard.security.AuthoritiesConstants;
import ai.giskard.service.InitService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@IntegrationTest
class ProjectConfigIT {
    @Autowired
    ApplicationProperties applicationProperties;

    @Autowired
    ProjectRepository projectRepository;

    @Autowired
    UserRepository userRepository;

    @Autowired
    MockMvc mockMvc;

    @Test
    @Transactional
    void getNumberSamplesDefaultProject() throws Exception {
        mockMvc.perform(get(String.format("/api/v2/project?key=%s", InitService.GERMAN_CREDIT_PROJECT_KEY))
                .accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON))
            .andExpect(jsonPath("$.inspectionSettings.limeNumberSamples").value(applicationProperties.getLimeNumberSamples()))
            .andExpect(jsonPath("$.key").value(InitService.GERMAN_CREDIT_PROJECT_KEY));
    }

    @Test
    @Transactional
    void modifiedNumberSamples() throws Exception {
        Project project = projectRepository.getOneByKey("credit");
        project.getInspectionSettings().setLimeNumberSamples(1000);
        projectRepository.save(project);
        projectRepository.saveAndFlush(project);
        mockMvc.perform(get(String.format("/api/v2/project?key=%s", InitService.GERMAN_CREDIT_PROJECT_KEY))
                .accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON))
            .andExpect(jsonPath("$.inspectionSettings.limeNumberSamples").value(1000))
            .andExpect(jsonPath("$.key").value(InitService.GERMAN_CREDIT_PROJECT_KEY));
    }
}
